import json
import re
import sys
from typing import Any, Dict, List


def extract_theorems_and_proofs(markdown_content: str) -> Dict[str, List[Dict[str, str]]]:
    """
    从Markdown内容中提取定理和证明部分
    
    参数:
        markdown_content: Markdown格式的文本内容
    
    返回:
        包含定理和证明的字典，格式为:
        {
            "概念": [],
            "定理": [
                {
                    "定理内容": "定理文本",
                    "证明": "证明文本"
                }
            ]
        }
    """
    # 定义正则表达式模式
    # 匹配定理编号和标题，如"2.1.17 LEMMA"或"### 2.1.18 THEOREM"
    theorem_pattern = re.compile(
        r'^(#*\s*)?(\d+．\d+．\d+) (定理|推论|引理)', 
        re.IGNORECASE | re.MULTILINE
    )
    
    # 匹配证明开始标记
    proof_pattern = re.compile(r'证', re.MULTILINE)
    
    # 匹配任何数字编号（如x.x.x格式）
    number_pattern = re.compile(r'^(#*\s*)?(\d+．\d+．\d+) (定理|推论|引理|定义|例|算法|注)', re.MULTILINE)


    definitions_pattern = re.compile(
        r'^(#*\s*)?(\d+．\d+．\d+) (定义|算法)', 
        re.IGNORECASE | re.MULTILINE
    )

    examples_pattern = re.compile(
        r'^(#*\s*)?(\d+．\d+．\d+) (例|注)', 
        re.IGNORECASE | re.MULTILINE
    )
    
    # 结果存储
    results = {
        "概念": [],
        "定理": [],
    }
    
    # 查找所有定理位置
    theorem_matches = list(theorem_pattern.finditer(markdown_content))
    
    for i, theorem_match in enumerate(theorem_matches):
        theorem_start = theorem_match.start()
        
        # 查找对应的证明开始位置
        proof_match = proof_pattern.search(markdown_content, theorem_start)
        if not proof_match:
            # 如果没有找到证明，跳过这个定理
            continue
        
        proof_start = proof_match.start()
        
        # 确定下一个数字编号的位置（如果存在）
        next_number_start = len(markdown_content)
        
        # 从当前定理位置之后查找下一个数字编号
        next_number_match = number_pattern.search(markdown_content, theorem_start + 2)
        if next_number_match:
            next_number_start = next_number_match.start()
        
        # 提取定理内容和证明内容
        theorem_content = markdown_content[theorem_start:next_number_start].strip()
        
        # 在定理编号和类型前后添加**
        theorem_content = theorem_pattern.sub(
            r'\1**\2 \3**', 
            theorem_content, 
            count=1  # 只替换第一个匹配（即定理标题）
        )
        
        # 添加到结果中
        results["定理"].append({
            "定理内容": theorem_content,
            "证明": ""
        })

    # 定义正则表达式模式
    # 匹配定理编号和标题，如"2.1.17 LEMMA"或"### 2.1.18 THEOREM"
    theorem_pattern = re.compile(
        r'^(#*\s*)?(\d+．\d+．\d+) (定理|推论|引理|定义|例|算法)', 
        re.IGNORECASE | re.MULTILINE
    )
        
    # 查找所有定理位置
    theorem_matches = list(theorem_pattern.finditer(markdown_content))
    
    for i, theorem_match in enumerate(theorem_matches):
        theorem_start = theorem_match.start()
        
        # 确定下一个数字编号的位置（如果存在）
        next_number_start = len(markdown_content)
        
        # 从当前定理位置之后查找下一个数字编号
        next_number_match = number_pattern.search(markdown_content, theorem_start + 2)
        if next_number_match:
            next_number_start = next_number_match.start()

        definition_match = definitions_pattern.search(markdown_content, theorem_start)
        example_match = examples_pattern.search(markdown_content, theorem_start)
        if (definition_match and definition_match.start() == theorem_start) or (example_match and example_match.start() == theorem_start):
                # 添加到结果中
                # 在定理编号和类型前后添加**
                theorem_content = markdown_content[theorem_start:next_number_start].strip()
                theorem_content = theorem_pattern.sub(
                    r'\1**\2 \3**', 
                    theorem_content, 
                    count=1  # 只替换第一个匹配（即定理标题）
                )
                if len(theorem_content.split("**")) > 1:
                    results["概念"].append({
                            "概念名": theorem_content.split("**")[1],
                            "概念定义": theorem_content
                        })
    
    return results

def main():
    """主函数，处理命令行参数并执行转换"""
    input_file = "数理统计讲义 (郑明,陈子毅,汪嘉冈) (Z-Library).md"
    output_file = "数理统计讲义 (郑明,陈子毅,汪嘉冈) (Z-Library).json"
    
    try:
        # 读取Markdown文件
        with open(input_file, 'r', encoding='utf-8') as f:
            markdown_content = f.read()
        
        # 提取定理和证明
        result = extract_theorems_and_proofs(markdown_content)
        
        # 写入JSON文件
        with open(output_file, 'w', encoding='utf-8') as f:
            json.dump(result, f, ensure_ascii=False, indent=2)
        
        print(f"成功提取 {len(result['定理'])} 个定理和证明到 {output_file}")
    
    except FileNotFoundError:
        print(f"错误: 文件 '{input_file}' 不存在")
        sys.exit(1)
    except Exception as e:
        print(f"发生错误: {e}")
        sys.exit(1)

if __name__ == "__main__":
    main()